
##########################################################
#
# File name: ps_input_json_creator.py
#
# Created on: Jun 27, 2019
#
# Author: Shamimuddin
#
##########################################################

import sys
#Kludge to make higher level directory work for using pyanalytics_lib.  This won't be needed when that actually gets into it's own package.
sys.path.append("..")

import time
from pathlib import Path
import simplejson as json
import time
import traceback
import os
from pynovatec import appconfig

from es.input_json_templator import get_LstSubAssemblyInstances, get_input_json_template, make_pa_input_json_init,get_LstSubAssemblyInstances

from pynovatec_compute.zmq.ZmqSender import JsonSender, InputJsonRSender, ZmqLogger
from dotenv import load_dotenv
from os import getenv as env

from pynovatec_compute.batchers import ZmqIntervalBatcher
from pynovatec_compute.batchers import CsvDataBatcher
from pynovatec.events import EventListener
from pynovatec_compute.batchers import RawBatchDataEvent
from es.BatchSaver import BatchSaver

load_dotenv('/etc/os-release')

default_config = {
  'host' : "https://api.machinesense.com/",
  'raw_data_recv_port' : "7778",
  'machine_details_dir_path' : "/opt/novatec/data/pumpsense/json_data",
  'machine_details_file_name' : "machine_details_0.json",
  'input_json_create_interval' : ["60", "600"],
  'input_json_send_port_to_r' : "5556",
  'input_json_send_port_to_json_sender' : "4445"
}

#conf_file_path = '/etc/prophecy/ps_input_json_creator_conf.json'
#conf_file_path = 'confs/input_json_creator.json'
conf_file_path = 'confs/input_json_creator.local.json'

class BatchTestListener(EventListener):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.events.append(RawBatchDataEvent)
        self.register()

    def handle(self,msg):
        print("BatchListener Handle Event: ",msg.data)
        raw_data_list = msg.data['data']
        duration = msg.data['duration']
        try:
          input_json = process_data_to_input_json(raw_data_list)
          if duration == 60:
              print("***************** Processing 1 minute data ***************************")
              json_filename = str(int(time.time())) + '.json'
              json_type = 1
          elif duration == 600:
              json_filename = str(int(time.time())) + '__' + input_json['header']['machineId'] + '.json'
              json_type = 10
 
          input_json_new = json.dumps(input_json)
          print("input_json_new: ",input_json)
          zmq_socket_jsonsender.send_string("input",json_filename,input_json_new)
          zmq_socket_rsender.send_string(json_type,input_json_new)
        except Exception as e:
          error_str = "BatchListener Exception: " + str(e)
          print(error_str)
          print(traceback.format_exc())


##########################################################
#
#                 Connect to Error Logger
#
##########################################################

def zmq_logger_init():
    appconfig.logger = ZmqLogger(connect_string = "tcp://127.0.0.1:6667")

def msg_log(msg):
    if hasattr(appconfig,"logger"): 
        appconfig.logger.log(msg)
    else:
        print(msg)


##########################################################
#
#                    Read conf json
#
##########################################################
def load_config():
    conf_file_exist = Path(conf_file_path)
    if conf_file_exist.is_file() == False:
         print('No such file ' + conf_file_path);
         msg_log('IJC~~~No such file ' + conf_file_path + ' ' + str(round(time.time())))
         sys.exit()
    with open (conf_file_path) as conf_json_file:
        conf_json_data = json.load(conf_json_file)
    appconfig.add(**conf_json_data)


def init_zmq_senders():
    global zmq_socket_jsonsender
    global zmq_socket_rsender

    # Connect to JSON Sender
    #zmq_socket_jsonsender = ZmqSender("tcp://127.0.0.1:" + appconfig.input_json_send_port_to_json_sender, language="input_json_creator")
    zmq_socket_jsonsender = JsonSender(connect_string = "tcp://127.0.0.1:" + appconfig.input_json_send_port_to_json_sender, process="input_json_creator")

    # Connect to R
    #zmq_socket_rsender = ZmqSender("tcp://127.0.0.1:" + appconfig.input_json_send_port_to_r)
    zmq_socket_rsender = InputJsonRSender(connect_string = "tcp://127.0.0.1:" + appconfig.input_json_send_port_to_r)


def process_data_to_input_json(data):
    print("************************** Processing INPUT JSON ***********************************")
    DictCSV = {'4': 'CSV_4', '6': 'CSV_6', '10': 'CSV_10', '20': 'CSV_20', '24': 'CSV_24', '28': 'CSV_28', '32': 'CSV_32'}
    input_json = get_input_json_template()
    LstSubAssemblyInstances=get_LstSubAssemblyInstances() 
    input_json['header']['timestamp'] = int(time.time())
    print("Starting input json: ",json.dumps(input_json,indent=4))
    print("LstSubAssemblyInstances: ",json.dumps(LstSubAssemblyInstances,indent=4))
    raw_data_list_len = len(data)
    for loop_counter in range (raw_data_list_len):
        raw_data_line = data[loop_counter]
        raw_data_line = raw_data_line.split(',')
        #(port,csv_id) = raw_data_line[1:3]
        #new_data_line = [ raw_data_line[0] ] + raw_data_line[3:]
        (port,csv_id) = raw_data_line[3:5]
        new_data_line = [ raw_data_line[0] ] + raw_data_line[6:]
        #del raw_data_line[1]
        #del raw_data_line[1]
        #print(f"Raw data is now({port},{csv_id}): {raw_data_line}") 
        # print(f"New Raw data is now({port},{csv_id}): {new_data_line}") 
        #input_json['subassemblies'][LstSubAssemblyInstances[int(port)]]['collectors']['pf1'][DictCSV.get(csv_id)][0].append(raw_data_line)
        if int(port) < len(LstSubAssemblyInstances):
            data_key = DictCSV.get(csv_id)
            if data_key in input_json['subassemblies'][LstSubAssemblyInstances[int(port)]]['collectors']['pf1']:
                input_json['subassemblies'][LstSubAssemblyInstances[int(port)]]['collectors']['pf1'][DictCSV.get(csv_id)][0].append(new_data_line)
    return input_json


def make_zmq_batchers(interval_list):
    batchers = []

    for interval in interval_list:
        print("interval: ",interval)
        batch_type = int(int(interval)/60)
        print("batch_type: ",batch_type)

        batcher = ZmqIntervalBatcher(interval=int(interval))
        batchers.append(batcher)
    return batchers

def make_csv_batchers(interval_list,base_path="/tmp/ramcsv"):
    batchers = []

    for interval in interval_list:
        print("interval: ",interval)
        batch_type = int(int(interval)/60)
        print("batch_type: ",batch_type)
        path=os.path.join(base_path,str(interval))
        print("Path: ",path)

        batcher = CsvDataBatcher(interval=int(interval),csv_path=path)
        batchers.append(batcher)
    return batchers

def es_main():
    
##########################################################
#
#                    Init threads
#
##########################################################

    msg_log('IJC~~~application started ' + str(round(time.time())))
    if appconfig.batch_mode == 'csv':
        batchers = make_csv_batchers(appconfig.input_json_create_interval,base_path=appconfig.ramcsv_path)
    else:
        batchers = make_zmq_batchers(appconfig.input_json_create_interval)
    #for interval in appconfig.input_json_create_interval:
    #    print("interval: ",interval)
    #    batch_type = int(int(interval)/60)
    #    print("batch_type: ",batch_type)

    #    batcher = ZmqIntervalBatcher(interval=int(interval))
    #    batchers.append(batcher)
    #    # batcher.start()

    l = BatchTestListener()

    if appconfig.batch_saver:
        saver=BatchSaver(appconfig.batch_saver)

##########################################################
#
#                    Start threads
#
##########################################################
    for th in batchers:
        th.start()

##########################################################
#
#               Wait for threads to join
#
##########################################################

    for th in batchers:
        th.join()
    #batcher.join()    

def run():
   init_zmq_senders()
   es_main()

def init(config=None):
#    global appconfig
   if config:
       appconfig=config
   else:
    load_config()

   make_pa_input_json_init(appconfig.machine_details_dir_path + '/' + appconfig.machine_details_file_name,custom_header={'host': appconfig.host})
   zmq_logger_init()

if __name__ == "__main__":
    
    init()
    run()
